#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-01-25
#
# add on for cisco support api. calls cisco bug api 2.0.
# for more information see: https://developer.cisco.com/docs/support-apis/#bug
#
# 2021-07-24: rewrite for python3.8
#
import logging
import ciscosupport
import ciscoapi
import os
import json
import time
import sys


def main():

    def get_new_token():
        response = ciscoapi.get_access_token()
        access_token = response.get('access_token', '')
        lifetime = int(time.time()) - int(response.get('lifetime', 3600)) - 30
        return access_token, lifetime

    ciscosupport.set_logging('debug')
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)
    base_path = '~/var/ciscoapi'
    conf_file = '~/etc/ciscoapi/ciscoapi.json'
    conf_file = os.path.expanduser(conf_file)

    access_token = ''
    lifetime = 0
    config = {}

    refresh_found = 2
    refresh_notfound = 1

    wait_after_start = True
    max_wait_time = 15

    loglevel = 'warning'

    if os.path.isfile(conf_file):
        with open(conf_file) as f:
            try:
                config = json.load(f)
            except ValueError as e:
                logging.warning(f'snmp_cisco_eox:status:JSON load error: {e}')
                exit()
        # global options
        base_path = config['global'].get('base_path', base_path)
        wait_after_start = config['global'].get('wait_after_start', wait_after_start)
        max_wait_time = config['global'].get('max_wait_time', max_wait_time)
        loglevel = config['global'].get('loglevel', loglevel)
        # bug api specific options
        refresh_found = config['bug'].get('refresh_found', refresh_found)
        refresh_notfound = config['bug'].get('refresh_notfound', refresh_notfound)
    else:
        logger.critical(f'Config file not found ({conf_file}).')
        exit()

    # loglevel = 'debug'
    ciscosupport.set_logging(loglevel)

    bug_path = base_path + '/bug'
    path_found = ciscosupport.expand_path(bug_path + '/found')
    path_not_found = ciscosupport.expand_path(bug_path + '/not_found')
    path_request = ciscosupport.expand_path(bug_path + '/request')
    path_missing = ciscosupport.expand_path(bug_path + '/missing')

    pids_requested = {}
    pids_refresh = {}
    pids = []

    # get list of bug reports to refresh
    pids = ciscosupport.get_subdirs_from_dir(path_found)
    for pid in pids:
        pids_refresh[pid.replace('_', '/')] = ','.join(ciscosupport.get_ids_from_dir(path_found+pid, refresh_time=refresh_found))

    # move not_found bug reports older then 'refresh_notfound' days to request (for refresh)
    ciscosupport.move_dir(path_not_found, path_request, refresh_time=refresh_notfound)

    # get list of PIDs requests
    pids = ciscosupport.get_subdirs_from_dir(path_request)
    logger.debug('bug requests (PIDs): %s' % pids)
    for pid in pids:
        pids_requested[pid.replace('_', '/')] = ','.join(ciscosupport.get_ids_from_dir(path_request+pid))
    logger.debug('bug requests (PIDs and releases): %s' % pids_requested)

    if len(pids_refresh.keys()) > 0 or len(pids_requested.keys()) > 0:

        reqoptions = []

        # modified_date
        # 1 = Last Week
        # 2 = Last 30 Days(default)
        # 3 = Last 6  Months
        # 4 = Last Year
        # 5 = All
        reqoptions.append('modified_date=1')
#        reqoptions.append('modified_date=3')  # for testing

        # severity (default is all)
        # 1 = Severity 1
        # 2 = Severity 2
        # 3 = Severity 3
        # 4 = Severity 4
        # 5 = Severity 5
        # 6 = Severity 6
        # reqoptions.append('severity=2')

        # status (default is all)
        # O = Open
        # F = Fixed
        # T = Terminated
        # reqoptions.append('status=O')

        # sort_by
        #  status
        #  modified_date (recent first, default)
        #  severity
        #  support_case_count
        #  modified_date_earliest (earliest first)
        # reqoptions.append('sort_by=severity')

        # reqoptions.append('page_index=2')

        if len(reqoptions) > 0:
            reqoptions = '?' + '&'.join(reqoptions)
        else:
            reqoptions = ''

        # wait random time after startup (load spread)
        if wait_after_start:
            ciscosupport.sleep_random(max_wait_time)

        # request access_token if no token available or if is expired
        if access_token == '':
            access_token, lifetime = get_new_token()

        if access_token == '':
            logger.critical('failed to get access_token')
            return

        # first refresh bug reports
        for pid in pids_refresh.keys():
            if lifetime > int(time.time()):
                access_token, lifetime = get_new_token()
            # get bug records for time frame
            bug_records = ciscoapi.get_bug_by_pid_and_release(pid, pids_refresh.get(pid), access_token, reqoptions)
            for entry in bug_records:
                pid = entry.get('pid')
                software_releases = entry.get('software_releases')
                status_code = int(entry.get('status_code', 200))
                logger.debug('bug return:PID: %s, Status: %s, Version: %s' % (pid, status_code, software_releases))
                path = ciscosupport.expand_path(path_found + pid.replace('/', '_'))
                # check if there where was an error, if so go to next pid
                if status_code == 200:
                    bugs = entry.get('bugs', None)
                    if bugs:  # if new/changed bugs found
                        for bug in bugs:
                            bug.pop('description')  # remove description
                        for release in software_releases.keys():  # create one bug list per software release
                            bug_release = software_releases.get(release)
                            logger.debug('bug found:PID: %s, Releases: %s' % (pid, release))
                            missing = entry.get('missing', {})
                            new_bugs = []
                            for bug in bugs:
                                if bug_release in bug.get('known_affected_releases'):
                                    new_bugs.append(bug)

                            # open found file
                            with open(path + release) as f:
                                try:
                                    bug_record = json.load(f)
                                except ValueError as e:
                                    logging.warning(f'{pid}:{release}:snmp_cisco_bug:bug_found:JSON load error: {e}')

                            found_bugs = bug_record.get('bugs')
                            for found_bug in found_bugs:
                                for new_bug in new_bugs:
                                    if found_bug.get('bug_id') == new_bug.get('bug_id'):
                                        found_bug.update(new_bug)  # update old bug with new data (changed bugs)
                                        new_bugs.remove(new_bug)
                            for bug in new_bugs:
                                found_bugs.append(bug)  # add new bugs
                            bug_record['bugs'] = found_bugs  # replace bug list with new list
                            bug_record['total_records'] = len(found_bugs)  # change number of records

                            # bug_record['missing'] = missing  # replace missing state with new missing state

                            with open(path + release, 'w') as f:
                                json.dump(bug_record, f)  # write updated bug report

                            # if len(missing.keys()) != 0:
                            #     path = ciscosupport.expand_path(path_missing + pid.replace('/', '_'))
                            #     with open(path + release, 'w') as f:
                            #         json.dump(missing, f)  # write update missing state

                    else:
                        for release in software_releases.keys():
                            if os.path.exists(path + release):
                                os.utime(path + release, None)

        reqoptions = []

        # modified_date
        # 1 = Last Week
        # 2 = Last 30 Days(default)
        # 3 = Last 6  Months
        # 4 = Last Year
        # 5 = All
        reqoptions.append('modified_date=5')
#        reqoptions.append('modified_date=2')  # for testing

        # severity (default is all)
        # 1 = Severity 1
        # 2 = Severity 2
        # 3 = Severity 3
        # 4 = Severity 4
        # 5 = Severity 5
        # 6 = Severity 6
        # reqoptions.append('severity=2')

        # status (default is all)
        # O = Open
        # F = Fixed
        # T = Terminated
        # reqoptions.append('status=O')

        # sort_by
        #  status
        #  modified_date (recent first, default)
        #  severity
        #  support_case_count
        #  modified_date_earliest (earliest first)
        # reqoptions.append('sort_by=severity')

        #reqoptions.append('page_index=2')

        if len(reqoptions) > 0:
            reqoptions = '?' + '&'.join(reqoptions)
        else:
            reqoptions = ''

        for pid in pids_requested.keys():
            if lifetime > int(time.time()):
                access_token, lifetime = get_new_token()
            bug_records = ciscoapi.get_bug_by_pid_and_release(pid, pids_requested.get(pid), access_token, reqoptions)
            for entry in bug_records:
                pid = entry.get('pid')
                software_releases = entry.get('software_releases')
                status_code = int(entry.get('status_code', 200))
                logger.debug('bug return:PID: %s, Status: %s, Version: %s' % (pid, status_code, software_releases))
                # check if there where was an error, if so go to next pid
                if status_code == 200:
                    bugs = entry.get('bugs', None)
                    if bugs:
                        for bug in bugs:
                            bug.pop('description')
                    for release in software_releases.keys():  # create one bug list per software release
                        bug_release = software_releases.get(release)
                        if bugs:
                            logger.debug('bug found:PID: %s, Releases: %s' % (pid, release))
                            missing = entry.get('missing', {})
                            # split bugs by release
                            release_bugs = {'pid': pid,
                                            'software_release': release,
                                            'bugs': [],
                                            'missing': missing}
                            for bug in bugs:
                                if bug_release in bug.get('known_affected_releases'):
                                    release_bugs['bugs'].append(bug)
                            # changed from entry.get('total_records') --> total records for all requested software releases
                            release_bugs['total_records'] = len(release_bugs['bugs'])

                            path = ciscosupport.expand_path(path_found + pid.replace('/', '_'))
                            with open(path + release, 'w') as f:
                                json.dump(release_bugs, f)

                            if len(missing.keys()) != 0:
                                path = ciscosupport.expand_path(path_missing + pid.replace('/', '_'))
                                with open(path + release, 'w') as f:
                                    json.dump(missing, f)
                        else:
                            logger.debug('bug not found:PID: %s, Version: %s' % (pid, release))
                            path = ciscosupport.expand_path(path_not_found + pid.replace('/', '_'))
                            logger.debug('not found: %s' % entry)
                            with open(path + release, 'w') as f:
                                json.dump(entry, f)
                                pass

                        # remove request file
                        try:
                            logger.debug('bug delete request:PID: %s, Version: %s' % (pid, release))
                            os.remove(path_request + pid.replace('/', '_') + '/' + release)
                        except OSError:
                            pass

        # clean up (remove empty directories)
        ciscosupport.remove_empty_sub_dirs(path_request)
        ciscosupport.remove_empty_sub_dirs(path_found)
        ciscosupport.remove_empty_sub_dirs(path_not_found)
        ciscosupport.remove_empty_sub_dirs(path_missing)

main()



