#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2017-05-15
#
#  https://developer.cisco.com/docs/support-apis/
#
# 2021-07-23: rewrite for python 3.8
#
import logging
import os
import json
import time
import ciscosupport
import ciscoapi
import sys


# split pids in known and unknown eox state
def split_pids(eoxr):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    eox_known = []
    eox_unkown = []

    for response in eoxr:
        EOXRecord = response.get('EOXRecord')
        #        PaginationResponseRecord = response.get('PaginationResponseRecord')
        #        PageIndex = PaginationResponseRecord.get('PageIndex')
        #        LastIndex = PaginationResponseRecord.get('LastIndex')
        #        PageRecords = PaginationResponseRecord.get('PageRecords')
        #        TotalRecords = PaginationResponseRecord.get('TotalRecords')
        if EOXRecord is not None:
            for PID in EOXRecord:
                if PID.get('EOLProductID') != '':
                    eox_known.append(PID)
                    logger.debug('EOLProductID  : %s' % PID.get('EOLProductID'))
                else:
                    eox_unkown.append(PID)
                    logger.debug('EOXInputValue : %s' % PID.get('EOXInputValue'))
    return {'eox_known': eox_known, 'eox_unknown': eox_unkown}


# split serials, expects a list of EoX Records from get EoX by serial
def split_serials(eoxr):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    serials = []

    for PID in eoxr:
        EOLProductID = PID.get('EOLProductID')
        logger.debug('serial split Eox: found PID: %s' % EOLProductID)
        EOXInputValue = PID.get('EOXInputValue').split(',')
        for serial in EOXInputValue:
            logger.debug('found Serial: %s' % serial)
            eox_serial = PID.copy()
            eox_serial.update({'EOXInputValue': serial})
            serials.append(eox_serial)
            logger.debug('Serial EoX: %s' % eox_serial)

    return serials


# save EoX records to file by PID,
# expects a list of EoX Records from Cisco EoX API 5.0,
# returns a list of saved PIDs
def save_eox(eox, path):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    saved = []

    for PID in eox:
        EOLProductID = PID.get('EOLProductID')
        # EOLProductID is empty for EoX Records with unknown EoX state (error or not announced)
        if EOLProductID == '':
            EOLProductID = PID.get('EOXInputValue')

        with open(path + (EOLProductID.replace('/','_')), 'w') as f:
            json.dump(PID, f)
            saved.append(EOLProductID)

    return saved


# save EoX records to file by serial number,
# expects a list of EoX Records from Cisco EoX API 5.0,
# returns a list of saved Serials
def save_serials(eox, path):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    saved = []

    for SERIAL in eox:
        EOXInputValue = SERIAL.get('EOXInputValue')
        if EOXInputValue != '':
            with open(path + EOXInputValue, 'w') as f:
                json.dump(SERIAL, f)
            saved.append(EOXInputValue)

    return saved


def main():

    ciscosupport.set_logging('debug')
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)
    base_path = '~/var/ciscoapi'
    conf_file = '~/etc/ciscoapi/ciscoapi.json'
    conf_file = os.path.expanduser(conf_file)

    access_token = ''
    lifetime = 0
    pids = []
    serials = []
    config = {}

    # refresh time in days
    eox_refresh_known = 31
    eox_refresh_unknown = 7

    wait_after_start = True
    max_wait_time = 15

    loglevel = 'warning'

    if os.path.isfile(conf_file):
        with open(conf_file) as f:
            try:
                config = json.load(f)
            except ValueError as e:
                logging.warning(f'snmp_cisco_eox:status:JSON load error: {e}')
                exit()

            base_path = config['global'].get('base_path', base_path)
            wait_after_start = config['global'].get('wait_after_start', wait_after_start)
            max_wait_time = config['global'].get('max_wait_time', max_wait_time)
            loglevel = config['global'].get('loglevel', loglevel)

            eox_refresh_known = config['eox'].get('eox_refresh_known', eox_refresh_known)
            eox_refresh_unknown = config['eox'].get('eox_refresh_unknown', eox_refresh_unknown)
    else:
        logger.critical('Config file not found (%s).' % conf_file)
        return False

    ciscosupport.set_logging(loglevel)

    eox_path = base_path + '/EoX'
    path_found = eox_path + '/found'
    path_not_found = eox_path + '/not_found'
    path_request = eox_path + '/request'

    path_request_pid = ciscosupport.expand_path(path_request + '/pid')
    path_found_pid = ciscosupport.expand_path(path_found + '/pid')
    path_not_found_pid = ciscosupport.expand_path(path_not_found + '/pid')

    path_request_ser = ciscosupport.expand_path(path_request + '/ser')
    path_found_ser = ciscosupport.expand_path(path_found + '/ser')
    path_not_found_ser = ciscosupport.expand_path(path_not_found + '/ser')

    # create list of PIDs to request EoX status for
    pids = ciscosupport.get_ids_from_dir(path_request_pid)
    logger.debug('pid requests : %s' % pids)
    # remove already known PIDs from list
    pids = ciscosupport.remove_ids_from_list(pids, path_found_pid)
    logger.debug('pid requests : %s' % pids)
    # remove PIDs already requested with unknown EoX status from list
    pids = ciscosupport.remove_ids_from_list(pids, path_not_found_pid)
    logger.debug('pid requests : %s' % pids)

    # refresh PIDs after 30 days by default
    pids = ciscosupport.refresh_ids_from_dir(path_not_found_pid, eox_refresh_unknown, pids, True)
    logger.debug('pid requests : %s' % pids)
    pids = ciscosupport.refresh_ids_from_dir(path_found_pid, eox_refresh_known, pids, False)
    logger.debug('pid requests : %s' % pids)

    # create list of serial numbers to request EoX status for
    serials = ciscosupport.get_ids_from_dir(path_request_ser)
    logger.debug('ser requests : %s' % serials)
    # remove already known serials from list
    serials = ciscosupport.remove_ids_from_list(serials, path_found_ser)
    logger.debug('ser requests : %s' % serials)
    # remove serials already requested with unknown EoX status from list
    serials = ciscosupport.remove_ids_from_list(serials, path_not_found_ser)
    logger.debug('ser requests : %s' % serials)

    # refresh serials after 30 days by default
    serials = ciscosupport.refresh_ids_from_dir(path_not_found_ser, eox_refresh_unknown, serials, True)
    logger.debug('ser requests : %s' % serials)
    serials = ciscosupport.refresh_ids_from_dir(path_found_ser, eox_refresh_known, serials, False)
    logger.debug('ser requests : %s' % serials)

    if pids == [] and serials == []:
        logger.debug('all list are empty. Do nothing.')
        return

    # wait random time after startup (load spread)
    if wait_after_start:
        ciscosupport.sleep_random(max_wait_time)

    # get time since epoch in seconds
    starttime = int(time.time())

    # request access_token if no token available or if is expired
    if (access_token == '') or lifetime < starttime:
        response = ciscoapi.get_access_token()
        access_token = response.get('access_token', '')
        lifetime = int(response.get('lifetime', 3600))

    lifetime = starttime + lifetime - 30

    if access_token == '':
        logger.critical('failed to get access_token')
        return

    if pids != []:
        eox = ciscoapi.get_eox_by_pid(pids, access_token)

        # split eox records in a list of known and unknown pid records
        eox = split_pids(eox)

        # save known pid reports
        pids = save_eox(eox.get('eox_known'), path_found_pid)
        # delete requests for known pids
        ciscosupport.remove_ids_from_dir(pids, path_request_pid)

        # save unknown pid reports
        pids = save_eox(eox.get('eox_unknown'), path_not_found_pid)
        # delete requests for unknown pids
        ciscosupport.remove_ids_from_dir(pids, path_request_pid)
        # delete pids from known were the status changed to unknown
        ciscosupport.remove_ids_from_dir(pids, path_found_pid)

    if serials != []:
        eox = ciscoapi.get_eox_by_serials(serials, access_token)
        logger.debug('eox by ser: %s' % eox)

        # split eox records in a list of known and unknown pid records
        eox = split_pids(eox)

        # split EoX records for known PIDs in one entry per serial
        serials = split_serials(eox.get('eox_known'))
        # save EoX records for serials with known EoX state
        serials = save_serials(serials, path_found_ser)
        logger.debug('EoX Serials: known: %s' % serials)
        # delete requests for known serials
        ciscosupport.remove_ids_from_dir(serials, path_request_ser)

        # split EoX records for unknown PIDs in one entry per serial
        serials = split_serials(eox.get('eox_unknown'))
        # save EoX records for serials with known EoX state
        serials = save_serials(serials, path_not_found_ser)
        # delete requests for unknown serials
        ciscosupport.remove_ids_from_dir(serials, path_request_ser)
        # delete serials from known were the status changed to unknown
        ciscosupport.remove_ids_from_dir(serials, path_found_ser)


main()
