#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2017-07-10
#
#  https://developer.cisco.com/docs/support-apis/
#
# 2018-06-06: fixed handling if state changes form found to notfound (delete old psirt found file)
# 2021-07-24: rewritten for python 3.8
#
#
import logging
import ntpath
import os
import json
import time
import ciscosupport
import ciscoapi
import sys
from typing import List
from dataclasses import dataclass


@dataclass
class Paths:
    found: str
    not_found: str
    request: str


@dataclass
class Refresh:
    found: int
    not_found: int


g_logger = None


def _psirt_remove_id_file(psirt_path: str, psirt_id: str):
    # set logg module name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    # delete psirt file
    try:
        logger.debug(f'delete psirt id file : {psirt_path + psirt_id}')
        os.remove(psirt_path + psirt_id)
    except OSError:
        pass


def _psirt_dump_record(psirt_record, psirt_id: str, psirt_path: str, psirt_path_request: str):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    with open(psirt_path + psirt_id, 'w') as f:
        json.dump(psirt_record, f)
    # delete request file
    _psirt_remove_id_file(psirt_path_request, psirt_id)

    return


def _check_psirt_record(psirt_record, psirt_id, psirt_path_found, psirt_path_request):
    """

    :param psirt_record:
    :param psirt_id:
    :param psirt_path_found:
    :param psirt_path_request:
    :returns:
    """
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    for advisory in psirt_record.get('advisories'):
        # remove unwanted information from advisories
        advisory.pop('productNames', None)
        advisory.pop('ipsSignatures', None)
        advisory.pop('iosRelease', None)
        advisory.pop('cvrfUrl', None)
        advisory.pop('ovalUrl', None)
        advisory.pop('summary', None)
        temp_advisory = advisory.copy()
        for key in temp_advisory.keys():
            if advisory.get(key, None) in [['NA'], 'NA']:
                advisory.pop(key, None)

    _psirt_dump_record(psirt_record, psirt_id, psirt_path_found, psirt_path_request)

    return


def _get_psirt_id_list(product_family: str, paths: Paths, refresh: Refresh) -> List[str]:
    """

    @param product_family:
    @param paths: Path object with path to founnd/not found/requested PSIRT records
    @param refresh: Refresh object with number of days before a PSIRT record needs to be refreshed for found/not found
    @return: list of PIDs
    """
    # create list of ID's to request PSIRT status for
    psirt_id_list = ciscosupport.get_ids_from_dir(paths.request + product_family)
    g_logger.debug(f'psirt requests : {psirt_id_list}')
    # remove already found ID's from list
    psirt_id_list = ciscosupport.remove_ids_from_list(psirt_id_list, paths.found + product_family)
    g_logger.debug(f'psirt requests : {psirt_id_list}')
    # remove not found ID's from list
    psirt_id_list = ciscosupport.remove_ids_from_list(psirt_id_list, paths.not_found + product_family)
    g_logger.debug(f'psirt requests : {psirt_id_list}')

    # refresh psirt after 1 day by default
    psirt_id_list = ciscosupport.refresh_ids_from_dir(paths.not_found + product_family, refresh.not_found,
                                                      psirt_id_list, True)
    g_logger.debug(f'psirt requests : {psirt_id_list}')
    psirt_id_list = ciscosupport.refresh_ids_from_dir(paths.found + product_family, refresh.found,
                                                      psirt_id_list, False)
    g_logger.debug(f'psirt requests : {psirt_id_list}')

    return psirt_id_list


def _update_psirt_id(psirt_records: list, family_name: str, paths: Paths):
    for psirt_record in psirt_records:
        if family_name in ['IOS', 'IOS-XE']:
            psirt_id = psirt_record.get('version')
        else:
            psirt_id = psirt_record.get('family')

        if psirt_record.get('advisories') != 'notfound':
            _check_psirt_record(psirt_record, psirt_id, paths.found + family_name + '/',
                                ntpath.sep + family_name + '/')
        else:
            _psirt_dump_record(psirt_record, psirt_id, paths.not_found + family_name + '/',
                               paths.request + family_name + '/')
            # remove psirt_found file (happens when product family is removed form bug ID)
            _psirt_remove_id_file(paths.found + family_name + '/', psirt_id)
    return


def main():
    global g_logger

    ciscosupport.set_logging('debug')
    # set logg modul name <file>:<module>.<function>
    g_logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    conf_file = '~/etc/ciscoapi/ciscoapi.json'
    conf_file = os.path.expanduser(conf_file)

    # default paths
    base_path = '~/var/ciscoapi'

    access_token = ''
    lifetime = 0
    config = {}

    # refresh time in days
    psirt_refresh_found = 1
    psirt_refresh_notfound = 1

    wait_after_start = True
    max_wait_time = 15
    loglevel = 'warning'
    # loglevel = 'debug'

    if os.path.isfile(conf_file):
        with open(conf_file) as f:
            try:
                config = json.load(f)
            except ValueError as e:
                logging.warning(f'snmp_cisco_eox:status:JSON load error: {e}')
                exit()

        base_path = config['global'].get('base_path', base_path)
        wait_after_start = config['global'].get('wait_after_start', wait_after_start)
        max_wait_time = config['global'].get('max_wait_time', max_wait_time)
        loglevel = config['global'].get('loglevel', loglevel)
        refresh = Refresh(
            found=config['psirt'].get('psirt_refresh_found', psirt_refresh_found),
            not_found=config['psirt'].get('psirt_refresh_notfound', psirt_refresh_notfound)
        )
    else:
        g_logger.critical(f'Config file not found ({conf_file}).')
        return False

    ciscosupport.set_logging(loglevel)

    base_path = ciscosupport.expand_path(base_path)
    psirt_dir = base_path + 'psirt'
    paths = Paths(
        found=psirt_dir + '/found/',
        not_found= psirt_dir + '/not_found/',
        request=psirt_dir + '/request/'
    )

    psirt_ios = _get_psirt_id_list('IOS', paths, refresh)
    psirt_ios_xe = _get_psirt_id_list('IOS-XE', paths, refresh)
    psirt_family = _get_psirt_id_list('family', paths, refresh)

    if (psirt_ios == []) and psirt_ios_xe == [] and psirt_family == []:
        g_logger.debug('all list are empty. Do nothing.')
        return

    # wait random time after startup
    if wait_after_start:
        ciscosupport.sleep_random(max_wait_time)

    # get time since epoch in seconds
    starttime = int(time.time())

    # request access_token if no token available or if is expired
    if (access_token == '') or lifetime < starttime:
        response = ciscoapi.get_access_token()
        access_token = response.get('access_token', '')
        lifetime = int(response.get('lifetime', 3600))

    lifetime = starttime + lifetime - 30

    if access_token == '':
        g_logger.critical('failed to get access_token')
        return

    if psirt_family != []:
        psirt_records = ciscoapi.get_psirt_by_product_family(psirt_family, access_token)
        _update_psirt_id(psirt_records, 'family', paths)

    if psirt_ios_xe != []:
        psirt_records = ciscoapi.get_psirt_by_iosxe_version(psirt_ios_xe, access_token)
        _update_psirt_id(psirt_records, 'IOS-XE', paths)

    if psirt_ios != []:
        psirt_records = ciscoapi.get_psirt_by_ios_version(psirt_ios, access_token)
        _update_psirt_id(psirt_records, 'IOS', paths)


main()
