#!/usr/bin/env python
# -*- encoding: utf-8; py-indent-offset: 4 -*-

#  https://developer.cisco.com/docs/support-apis/
#
# 2017-03-30: Cisco EoX API Framework
#             added support for Cisco SN2INFO API (get contract status)
# 2017-07-09: added support for Cisco Psirt API (IOS and IOSXE)
# 2017-07-19: added support for Cisco Software Suggestion API
# 2018-01-25: adding support for Cisco bug api 2.0
# 2018-09-25: performance improvement "psirt_response.encoding = 'UTF-8'", drops json.loads
#             from about 4 min to some seconds
# 2021-07-23: rewritten for python 3.8
#
import requests
import json
import time
import logging
from os.path import isfile
from os.path import expanduser
import urllib
import sys

# global variables
g_client_id = ''
g_client_secret = ''
g_proxies = {}
g_use_system_proxies = True
g_useauthproxy = False
g_client_fqdn = ''
g_root_cert = True
g_authproxyurl = 'https://cmk.bech-noc.de/api/cauthproxy.py'


def read_config():
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    # define global variables
    global g_client_id
    global g_client_secret
    global g_proxies
    global g_use_system_proxies
    global g_useauthproxy
    global g_client_fqdn
    global g_root_cert
    global g_authproxyurl

    # read parameters from config file
    conf_file = '~/etc/ciscoapi/ciscoapi.json'
    conf_file = expanduser(conf_file)

    if isfile(conf_file):
        with open(conf_file) as f:
            try:
                config = json.load(f)
            except ValueError as e:
                logging.warning(f'snmp_cisco_eox:status:JSON load error: {e}')
                exit()

        g_client_id = config['cisco_api']['client_id']
        g_client_secret = config['cisco_api']['client_secret']
        if config['global'].get('http_proxy'):
            g_proxies.update({'http': config['global'].get('http_proxy')})
        if config['global'].get('https_proxy'):
            g_proxies.update({'https': config['global'].get('https_proxy')})
        g_use_system_proxies = (config['global'].get('use_system_proxies', False))
        # parameters for authproxy
        g_useauthproxy = config['cisco_api']['useauthproxy']
        if g_useauthproxy:
            g_client_fqdn = config['cisco_api'].get('client_fqdn')
            g_root_cert = config['cisco_api'].get('root_cert')
            g_authproxyurl = config['cisco_api'].get('authproxyurl')
    else:
        logger.critical('Config file not found (%s).' % conf_file)
        return False

    #if g_proxies == {}:
    #    if g_use_system_proxies:
    #        g_proxies = urllib.getproxies()

    return True


# Request Cisco API access token, token is valid for 1 hour (by default)
def get_access_token():
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    # define global variables
    global g_client_id
    global g_client_secret
    global g_proxies
    global g_useauthproxy
    global g_client_fqdn
    global g_root_cert
    global g_authproxyurl

    # define local variables
    access_token = ''
    lifetime = 0
    grant_type = 'client_credentials'
    authheaders = {'content-Type': 'application/x-www-form-urlencoded', 'accept': 'application/json'}
    authurl = 'https://cloudsso.cisco.com/as/token.oauth2'
    param_missing = False
    verify = True

    if read_config():
        if g_client_id == '':
            logging.critical('Option client_id missing.')
            param_missing = True
        if g_client_secret == '':
            logging.critical('Option client_secret missing.')
            param_missing = True
        # parameters for authproxy
        if g_useauthproxy:
            # if not g_root_cert:
            #     g_root_cert = expanduser(g_root_cert)
            #     if not isfile(g_root_cert):
            #         logging.critical('Root Cert not found: %s' % g_root_cert)
            #         param_missing = True
            if g_authproxyurl == '':
                logging.critical('Option authproxyurl missing')
                param_missing = True

    else:
        return {'access_token': access_token, 'lifetime': lifetime}

    if param_missing:
        return {'access_token': access_token, 'lifetime': lifetime}

    authreqdata = {'client_id': g_client_id, 'client_secret': g_client_secret, 'grant_type': grant_type}

    # Disable invalid certificate warnings.
    # requests.packages.urllib3.disable_warnings()

    # modify request if we use an authentication proxy
    if g_useauthproxy:
        if g_client_fqdn != '':
            authreqdata.update({'client_fqdn': g_client_fqdn})
        verify = g_root_cert
        authurl = g_authproxyurl

    logger.debug('auth request: authurl=%s headers=%s data=%s proxies=%s verify=%s' % (authurl, authheaders, authreqdata, g_proxies, verify))
    response = requests.post(authurl, headers=authheaders, data=authreqdata, proxies=g_proxies, verify=verify)

    logger.debug('response.text: %s' % response.text)
    if response.ok:
        authresponse = json.loads(response.text)
        access_token = authresponse.get("access_token")
        # access_token = access_token.decode('ascii')
        lifetime = int(authresponse.get("expires_in"))

    return {'access_token': access_token, 'lifetime': lifetime}


# generic cisco api request for all get info by serialnumber
def get_info_by_serials(serials, access_token, requrl, max_serials):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    # define global variables
    global g_proxies

    # locale variablen
    max_serial_length = 40
    max_req_per_second = 5
    wait_time = 5
    optimisedserials = []
    serialsstr = ''
    count = 1
    info = []

    # read config, we need only the proxies
    read_config()

    # split list of Serials in chunks of max 75 serials, each max 40 bytes length
    for serial in serials:
        if len(serial) <= max_serial_length:
            serialsstr += serial + ","
            count += 1
        if count == max_serials:
            optimisedserials.append(serialsstr[:-1])
            serialsstr = ''
            count = 1
    optimisedserials.append(serialsstr[:-1])

    headers = {'accept': 'application/json', 'Authorization': 'Bearer ' + access_token}

    count = 0
    for serials in optimisedserials:
        # Disable invalid certificate warnings.
        # requests.packages.urllib3.disable_warnings()
        apiresponse = requests.get(requrl + serials, headers=headers, proxies=g_proxies)
        count += 1
        # only 5 request per second are allowed
        if count == max_req_per_second:
            time.sleep(wait_time)
            count = 0
        if apiresponse.ok:
            apiresponse.encoding = 'UTF-8'
            info.append(json.loads(apiresponse.text))

    return info


def get_eox_by_pid(pids, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    # define global variables
    global g_proxies

    # local variables
    max_pid_length = 240
    max_pids = 20
    max_req_per_second = 5
    wait_time = 5
    optimisedpids = []
    pidstr = ''
    count = 1
    eoxr = []

    # read config, we need only the proxies
    read_config()

    # split list of PIDs in chunks of max 240 bytes length
    for pid in pids:
        if (len(pidstr) + len(pid)) < max_pid_length:
            pidstr += pid + ","
            count += 1
        if (count == max_pids) or ((len(pidstr) + len(pid)) >= max_pid_length):
            optimisedpids.append(pidstr[:-1])
            pidstr = ''
            count = 1
    optimisedpids.append(pidstr[:-1])

    headers = {'accept': 'application/json', 'Authorization': f'Bearer {access_token}'}
    requrl = 'https://api.cisco.com/supporttools/eox/rest/5/EOXByProductID/1/'

    count = 0
    for productids in optimisedpids:
        # Disable invalid certificate warnings.
        # requests.packages.urllib3.disable_warnings()
        eoxresponse = requests.get(requrl + productids, headers=headers, proxies=g_proxies)
        count += 1
        # only 5 request per second are allowed
        if count == max_req_per_second:
            time.sleep(wait_time)
            count = 0
        if eoxresponse.ok:
            eoxr.append(json.loads(eoxresponse.text))
    return eoxr


def get_eox_by_serials(serials, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)
    max_serials = 20
    requrl = 'https://api.cisco.com/supporttools/eox/rest/5/EOXBySerialNumber/1/'
    info = get_info_by_serials(serials, access_token, requrl, max_serials)
    return info


def get_coverage_status_by_serials(serials, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)
    max_serials = 75
    requrl = 'https://api.cisco.com/sn2info/v2/coverage/status/serial_numbers/'
    info = get_info_by_serials(serials, access_token, requrl, max_serials)
    return info


def get_coverage_summary_by_serials(serials, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)
    max_serials = 75
    requrl = 'https://api.cisco.com/sn2info/v2/coverage/summary/serial_numbers/'
    info = get_info_by_serials(serials, access_token, requrl, max_serials)
    return info


def get_orderable_pid_by_serials(serials, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)
    max_serials = 75
    requrl = 'https://api.cisco.com/sn2info/v2/identifiers/orderable/serial_numbers/'
    info = get_info_by_serials(serials, access_token, requrl, max_serials)
    return info


def get_owner_coverage_status_by_serials(serials, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)
    max_serials = 75
    requrl = 'https://api.cisco.com/sn2info/v2/coverage/owner_status/serial_numbers/'
    info = get_info_by_serials(serials, access_token, requrl, max_serials)
    return info


def get_psirt_by_ios_version(psirtios, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    info = []

    # read config, we need only the proxies
    read_config()

    headers = {'accept': 'application/json', 'Authorization': 'Bearer ' + access_token}
    requrl = 'https://api.cisco.com/security/advisories/ios?version='

    # requests.packages.urllib3.disable_warnings()
    if list(psirtios) != []:
        for ios_version in psirtios:
            logger.debug('request          ios_version: %s, time: %s' % (ios_version, time.asctime(time.localtime(time.time()))))
            psirt_response = requests.get(requrl + ios_version, headers=headers, proxies=g_proxies)
            if psirt_response.ok:
                logger.debug('ok               ios_version: %s, time: %s, len: %s' % (ios_version, time.asctime(time.localtime(time.time())), len(str(psirt_response))))
                psirt_response.encoding = 'UTF-8'  # makes json.loads() mutch more faster (from 4 min. down to 1 sec for about 2MB)
                response = (json.loads(psirt_response.text))
                logger.debug('response loaded: ios_version: %s, time: %s, len: %s' % (ios_version, time.asctime(time.localtime(time.time())), len(str(response))))
                info.append({'version': ios_version, 'advisories': response.get('advisories', 'notfound')})
                logger.debug('ciscoapi:psirt-ios-found: %s' % info)
            else:
                logger.debug('notfound         ios_version: %s, time: %s, len: %s' % (ios_version, time.asctime(time.localtime(time.time())), len(str(psirt_response))))
                info.append({'version': ios_version, 'advisories': 'notfound'})
                logger.debug('ciscoapi:psirt-ios-notfound: %s' % info)
    return info


def get_psirt_by_iosxe_version(psirtios, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    info = []

    # read config, we need only the proxies
    read_config()

    headers = {'accept': 'application/json', 'Authorization': 'Bearer ' + access_token}
    requrl = 'https://api.cisco.com/security/advisories/iosxe?version='

    # requests.packages.urllib3.disable_warnings()
    if list(psirtios) != []:
        for ios_version in psirtios:
            psirt_response = requests.get(requrl + ios_version, headers=headers, proxies=g_proxies)
            if psirt_response.ok:
                psirt_response.encoding = 'UTF-8'
                response = (json.loads(psirt_response.text))
                info.append({'version': ios_version, 'advisories': response.get('advisories', 'notfound')})
                logger.debug('ciscoapi:psirt-iosxe-found: %s' % info)
            else:
                info.append({'version': ios_version, 'advisories': 'notfound'})
                logger.debug('ciscoapi:psirt-iosxe-notfound: %s' % info)
    return info


def get_psirt_by_product_family(families, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    info = []
    # read config, we need only the proxies
    read_config()

    headers = {'accept': 'application/json', 'Authorization': 'Bearer ' + access_token}
    requrl = 'https://api.cisco.com/security/advisories/cvrf/product?product='

    # requests.packages.urllib3.disable_warnings()
    if list(families) != []:

        for family in families:
            psirt_response = requests.get(requrl + family, headers=headers, proxies=g_proxies)
            if psirt_response.ok:
                psirt_response.encoding = 'UTF-8'
                response = (json.loads(psirt_response.text))
                info.append({'family': family, 'advisories': response.get('advisories', 'notfound')})
                logger.debug('ciscoapi:psirt-family-found: %s' % info)
            else:
                info.append({'family': family, 'advisories': 'notfound'})
                logger.debug('ciscoapi:psirt-family-notfound: %s' % info)
    return info


def get_suggested_image_and_release_by_pid(pids, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    info = []

    # read config, we need only the proxies
    read_config()

    headers = {'accept': 'application/json', 'Authorization': 'Bearer ' + access_token}
    # requrl = 'https://api.cisco.com/software/suggestion/v1.0/suggestions/productid/'
    requrl = 'https://api.cisco.com/software/suggestion/v2/suggestions/software/productIds/'

    # requests.packages.urllib3.disable_warnings()
    if list(pids) != []:
        for pid in pids:
            suggestion_response = requests.get(requrl + pid, headers=headers, proxies=g_proxies)
            if suggestion_response.ok:
                suggestion_response.encoding = 'UTF-8'
                response = (json.loads(suggestion_response.text))
                logger.debug('cisco-api:get_suggested_image_and_release_by_pid:response: %s' % response)
                info.append({'pid': pid, 'suggetion': response.get('advisories', 'notfound')})
                logger.debug('ciscoapi:info-found: %s' % info)
            else:
                info.append({'pid': pid, 'advisories': 'notfound'})
                logger.debug('ciscoapi:info-notfound: %s' % info)
    return info


def get_suggested_release_by_pid(pids, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    max_pid_length = 240
    max_pids = 10
    max_req_per_second = 5
    wait_time = 5
    optimisedpids = []
    pidstr = ''
    count = 1
    info = []

    # read config, we need only the proxies
    read_config()

    # split list of PIDs in chunks of max 240 bytes length
    for pid in pids:
        if (len(pidstr) + len(pid)) < max_pid_length:
            pidstr += pid + ","
            count += 1
        if (count == max_pids) or ((len(pidstr) + len(pid)) >= max_pid_length):
            optimisedpids.append(pidstr[:-1])
            pidstr = ''
            count = 1
    optimisedpids.append(pidstr[:-1])

    headers = {'accept': 'application/json', 'Authorization': 'Bearer ' + access_token}
    # requrl = 'https://api.cisco.com/software/suggestion/v1.0/suggestions/releases/'
    requrl = 'https://api.cisco.com/software/suggestion/v2/suggestions/releases/productIds/'

    # requests.packages.urllib3.disable_warnings()
    if optimisedpids != []:
        for productids in optimisedpids:
            suggestion_response = requests.get(requrl + productids, headers=headers, proxies=g_proxies)
            logger.debug('cisco-api:get_sugessted_release_by_pid:response:response.text %s' % suggestion_response.text)
            if suggestion_response.ok:
                suggestion_response.encoding = 'UTF-8'
                response = (json.loads(suggestion_response.text))
                logger.debug('cisco-api:get_suggested_release_by_pid:response: %s' % response)
                info.append(response.get('productList', 'notfound'))
                logger.debug('ciscoapi:info-found: %s' % info)

    return info


def get_sugessted_compatible_release_by_pid(pids, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    return


# get_clean_sn_for_bug_api('ASA5510', '9.1(7)15,8.4(7)30,9.1(6)1')
# return {'9.1(7)15': '9.1(7.15)', '9.1(6)1': '9.1(6.1)', '8.4(7)30': '8.4(7.30)'}
def get_clean_sn_for_bug_api(pid, software_releases):
    if software_releases == '' or pid == '':
        return {}
    software_releases = software_releases.split(',')
    clean_sns = {}

    if pid.startswith('ASA'):
        # change ASA version from 9.1(2)10 to 9.1(2.10)
        for software_release in software_releases:
            clean_sn = software_release
            if clean_sn[-1] != ')':
                clean_sn = clean_sn.split(')')
                if len(clean_sn) == 2:
                    clean_sn = '%s.%s)' % (clean_sn[0], clean_sn[1])
            clean_sns.update({software_release: clean_sn})

    elif pid.startswith('AIR'):
        # change WLC version from 8.5.120.0 to 8.5(120.0)
        for software_release in software_releases:
            clean_sn = software_release
            if clean_sn[-1] != ')':
                if len(clean_sn.split('.')) == 4:
                    clean_sn = clean_sn.split('.')
                    clean_sn = '%s.%s(%s.%s)' % (clean_sn[0], clean_sn[1], clean_sn[2], clean_sn[3])
            clean_sns.update({software_release: clean_sn})

    else:
        for software_release in software_releases:
            clean_sn = software_release
            if clean_sn.startswith('16.0'):
                # remove leading zeros 16.09.01 to 16.9.1
                clean_sn = clean_sn.split('.')
                for x in range(0, len(clean_sn)):
                    clean_sn[x] = clean_sn[x].lstrip('0')
                    if clean_sn[x] == '':
                        clean_sn = '0'
                clean_sn = '.'.join(clean_sn)
                # remove trailing A-Za-z
                while clean_sn[-1].upper() in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ':
                    clean_sn = clean_sn[:-1]
            elif clean_sn.startswith('03.0'):
                # convert version from 03.03.04SE to 3.3(4)SE
                clean_sn = clean_sn.split('.')
                for x in range(0, len(clean_sn)):
                    clean_sn[x] = clean_sn[x].lstrip('0')
                    if clean_sn[x] == '':
                        clean_sn = '0'
                digits = ''
                for x in clean_sn[2]:
                    if x.isdigit():
                        digits += x
                clean_sn[2] = clean_sn[2].replace(digits, '')
                clean_sn = '%s.%s(%s)%s' % (clean_sn[0], clean_sn[1], digits, clean_sn[2])
            clean_sns.update({software_release: clean_sn})

    return clean_sns


def get_bug_by_pid_and_release(pid, release, access_token, reqoptions):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    info = []

    # read config, we need only the proxies
    read_config()
    headers = {'accept': 'application/json', 'Authorization': 'Bearer ' + access_token}
    requrl = 'https://api.cisco.com/bug/v2.0/bugs/products/product_id/'

    if release != '':
        software_releases = get_clean_sn_for_bug_api(pid, release)
        clean_sn = ','.join(software_releases.values())
        missing = {}
        bug_response = requests.get(requrl + '%s/software_releases/%s%s' % (pid, clean_sn, reqoptions), headers=headers, proxies=g_proxies)
        if bug_response.ok:
            bug_response.encoding = 'UTF-8'
            response = (json.loads(bug_response.text))
            bug_list = response.get('bugs')
            pagination_record = response.get('pagination_response_record')
            last_page = int(pagination_record.get('last_index'))
            total_records = int(pagination_record.get('total_records'))
            logger.info('PID: %s, Version: %s, Total records: %s, Pages: %s' % (pid, clean_sn, total_records, last_page))
            if last_page > 1:
                if reqoptions != '':
                    reqoptions = reqoptions + '&page_index='
                else:
                    reqoptions = '?page_index='
                for page in range(2, last_page + 1):
                    # time.sleep(2)
                    page_options = reqoptions + '%s' % page
                    bug_response = requests.get(requrl + '%s/software_releases/%s%s' % (pid, clean_sn, page_options), headers=headers, proxies=g_proxies)
                    if bug_response.ok:
                        response = (json.loads(bug_response.text))
                        bug_list += response.get('bugs')
                    else:
                        bug_response.encoding = 'UTF-8'
                        status_code = bug_response.status_code
                        reason = bug_response.reason
                        url = bug_response.url
                        text = bug_response.text
                        logging.warning('ciscoapi error: %s, %s, Page: %s, LastPage: %s, URL: \'%s\'. Text: \'%s\'' % (status_code, reason, page, last_page, url, text))
                        missing.update({page: {'status_code': status_code,
                                               'reason': reason,
                                               'url': url,
                                               'text': text}})
            info.append({'pid': pid,
                         'software_releases': software_releases,
                         'bugs': bug_list,
                         'total_records': total_records,
                         'missing': missing})
            logger.debug('ciscoapi:bug-found: %s' % info)
        else:
            bug_response.encoding = 'UTF-8'
            status_code = bug_response.status_code
            reason = bug_response.reason
            url = bug_response.url
            text = bug_response.text
            page = 'ALL'
            logging.warning('ciscoapi error: %s, %s, URL: \'%s\'. Text: \'%s\'' % (status_code, reason, url, text))
            missing.update({page: {'status_code': status_code,
                                   'reason': reason,
                                   'url': url,
                                   'text': text}})
            info.append({'pid': pid,
                         'software_releases': software_releases,
                         'status_code': status_code,
                         'reason': reason,
                         'missing': missing})
    return info


def get_bug_by_productseries_and_affected_release(ps_release, access_token, reqoptions):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    info = []

    # read config, we need only the proxies
    read_config()
    headers = {'accept': 'application/json', 'Authorization': 'Bearer ' + access_token}
    requrl = 'https://api.cisco.com/bug/v2.0/bugs/product_series/'

    if len(ps_release.keys()) > 0:
        for product_series in ps_release.keys():
            affected_releases = ps_release.get(product_series).split(',')
            for affected_release in affected_releases:
                bug_response = requests.get(requrl + '%s/affected_releases/%s%s' % (product_series, affected_release, reqoptions), headers=headers, proxies=g_proxies)
                if bug_response.ok:
                    bug_response.encoding = 'UTF-8'
                    response = (json.loads(bug_response.text))
                    last_page = int(response.get('pagination_response_record').get('last_index'))
                    total_records = int(response.get('pagination_response_record').get('total_records'))
                    logger.debug('Product series: %s, Version: %s, Total records: %s, Pages: %s' % (product_series, affected_release, total_records, last_page))

                    info.append({'produckt_series': product_series, 'affected_releases': affected_releases, 'bugs': response})
                    logger.debug('ciscoapi:bug-found: %s' % info)
                else:
                    info.append({'produckt_series': product_series, 'affected_releases': affected_releases, 'bugs': 'notfound'})
                    logger.debug('ciscoapi:bug-not-found: %s' % info)

    return info


def get_product_mdf_information_by_pid(pids, access_token):
    # set logg modul name <file>:<module>.<function>
    logger = logging.getLogger(__file__ + ':' + __name__ + '.' + sys._getframe().f_code.co_name)

    max_pid_length = 40
    max_pids = 5
    max_req_per_second = 2
    wait_time = 5
    optimisedpids = []
    pidstr = ''
    count = 1
    info = []

    # read config, we need only the proxies
    read_config()

    # split list of PIDs in chunks of max 240 bytes length
    for pid in pids:
        if (len(pidstr) + len(pid)) < max_pid_length:
            pidstr += pid + ","
            count += 1
        if (count == max_pids) or ((len(pidstr) + len(pid)) >= max_pid_length):
            optimisedpids.append(pidstr[:-1])
            pidstr = ''
            count = 1
    optimisedpids.append(pidstr[:-1])

    headers = {'accept': 'application/json', 'Authorization': 'Bearer ' + access_token}
    requrl = 'https://api.cisco.com/product/v1/information/product_ids_mdf/'
#    requrl = 'https://api.cisco.com/product/v1/information/product_ids/'

    # requests.packages.urllib3.disable_warnings()
    if optimisedpids != []:
        for productids in optimisedpids:
            productinfo_response = requests.get(requrl + productids, headers=headers, proxies=g_proxies)
            logger.debug('cisco-api:get_product_mdf_information_by_pid:response:response.text %s' % productinfo_response.text)
            if productinfo_response.ok:
                productinfo_response.encoding = 'UTF-8'
                response = (json.loads(productinfo_response.text))
                logger.debug('cisco-api:get_product_mdf_information_by_pid:response: %s' % response)
                # info.append(response)
                info.extend(response.get('product_list', 'notfound'))
                logger.debug('ciscoapi:info-found: %s' % info)

    return info